@@ -12,7 +12,7 @@ from http.client import HTTPResponse
 from bottle import abort, get, post, put, delete, local, redirect, request, response
 from codalab.bundles import get_bundle_subclass
 from codalab.bundles.uploaded_bundle import UploadedBundle
-from codalab.common import StorageType, precondition, UsageError, NotFoundError
+from codalab.common import StorageType, StorageFormat, precondition, UsageError, NotFoundError
 from codalab.lib import canonicalize, spec_util, worksheet_util, bundle_util
 from codalab.lib.beam.filesystems import LOCAL_USING_AZURITE
 from codalab.lib.server_util import (
@@ -34,9 +34,12 @@ from codalab.objects.permission import (
 from codalab.rest.schemas import (
     BundleSchema,
     BundlePermissionSchema,
+    BundleLocationSchema,
+    BundleLocationListSchema,
     BUNDLE_CREATE_RESTRICTED_FIELDS,
     BUNDLE_UPDATE_RESTRICTED_FIELDS,
     WorksheetSchema,
+    BundleStoreSchema,
 )
 from codalab.rest.users import UserSchema
 from codalab.rest.util import get_bundle_infos, get_resource_ids, resolve_owner_in_keywords
@@ -426,6 +429,161 @@ def _fetch_locations():
     return dict(data=uuids_to_locations)
 
 
+@get(
+    '/bundles/<bundle_uuid:re:%s>/locations/' % spec_util.UUID_STR,
+    apply=AuthenticatedProtectedPlugin(),
+)
+def _fetch_bundle_locations(bundle_uuid: str):
+    """
+    Returns a list of BundleLocations associated with the given bundle.
+
+    Query parameters:
+    - `bundle_uuid`: Bundle UUID to get the locations for
+    """
+    bundle_locations = local.model.get_bundle_locations(bundle_uuid)
+
+    return BundleLocationListSchema(many=True).dump(bundle_locations).data
+
+
+@post(
+    '/bundles/<bundle_uuid:re:%s>/locations/' % spec_util.UUID_STR,
+    apply=AuthenticatedProtectedPlugin(),
+)
+def _add_bundle_location(bundle_uuid: str):
+    """
+    Adds a new BundleLocation to a bundle. Request body must contain the fields in BundleLocationSchema.
+
+    Query parameters:
+    - `bundle_uuid`: Bundle UUID corresponding to the new location
+    """
+    new_location = BundleLocationSchema(many=True).load(request.json).data[0]
+    new_location["uuid"] = local.model.add_bundle_location(
+        new_location['bundle_uuid'], new_location['bundle_store_uuid']
+    )
+    return BundleLocationSchema(many=True).dump([new_location]).data
+
+
+@get(
+    '/bundles/<bundle_uuid:re:%s>/locations/<bundle_store_uuid:re:%s>/',
+    apply=AuthenticatedProtectedPlugin(),
+)
+def _fetch_bundle_location(bundle_uuid: str, bundle_store_uuid: str):
+    """
+    Get info about a specific BundleLocation.
+
+    Query parameters:
+    - `bundle_uuid`: Bundle UUID to get the location for
+    - `bundle_store_uuid`: Bundle Store UUID to get the location for
+    """
+    bundle_location = local.model.get_bundle_location(bundle_uuid, bundle_store_uuid)
+    return BundleLocationListSchema(many=True).dump(bundle_location).data
+
+
+@get('/bundle_stores', apply=AuthenticatedProtectedPlugin())
+def _fetch_bundle_stores():
+    """
+    Fetch the bundle stores available to the user. No required arguments.
+
+    Returns a list of bundle stores, each having the following parameters:
+    - `uuid`: bundle store UUID
+    - `owner_id`: (integer) owner of bundle store
+    - `name`: name of bundle store
+    - `storage_type`: type of storage being used for bundle store (GCP, AWS, etc)
+    - `storage_format`: the format in which storage is being stored (UNCOMPRESSED, COMPRESSED_V1, etc)
+    - `url`: a self-referential URL that points to the bundle store.
+    """
+    bundle_stores = local.model.get_bundle_stores(request.user.user_id)
+    return BundleStoreSchema(many=True).dump(bundle_stores).data
+
+
+@post('/bundle_stores', apply=AuthenticatedProtectedPlugin())
+def _add_bundle_store():
+    """
+    Add a bundle store that the user can access.
+    JSON parameters:
+        - `name`: name of bundle store
+        - `storage_type`: type of storage being used for bundle store (GCP, AWS, etc)
+        - `storage_format`: the format in which storage is being stored (UNCOMPRESSED, COMPRESSED_V1, etc). If unspecified, an optimal default will be set.
+        - `url`: a self-referential URL that points to the bundle store.
+        - `authentication`: key for authentication that the bundle store uses.
+    Returns the data of the created bundle store.
+    """
+    new_bundle_store = BundleStoreSchema(strict=True, many=True).load(request.json).data[0]
+    storage_type = new_bundle_store.get('storage_type')
+    storage_format = new_bundle_store.get('storage_format')
+    if storage_format is None:
+        if storage_type in (StorageType.AZURE_BLOB_STORAGE.value, StorageType.GCS_STORAGE.value):
+            storage_format = StorageFormat.COMPRESSED_V1.value
+        elif storage_type == StorageType.DISK_STORAGE.value:
+            storage_format = StorageFormat.UNCOMPRESSED.value
+        else:
+            raise UsageError(
+                f"Could not determine default storage format for storage type {storage_type}."
+            )
+    uuid = local.model.create_bundle_store(
+        request.user.user_id,
+        new_bundle_store.get('name'),
+        storage_type,
+        storage_format,
+        new_bundle_store.get('url'),
+        new_bundle_store.get('authentication'),
+    )
+    bundle_store = local.model.get_bundle_store(request.user.user_id, uuid)
+    return BundleStoreSchema(many=True).dump([bundle_store]).data
+
+
+# TODO: Endpoint not tested / used, reenable when we use it.
+# @put('/bundle_stores/<uuid:re:%s>' % spec_util.UUID_STR, apply=AuthenticatedProtectedPlugin())
+# def _update_bundle_store(uuid):
+#     """
+#     Update a bundle store that the user can access.
+#     Query Parameters:
+#     - `uuid`: uuid of bundle store
+#     JSON Parameters (needs to contain at least one of these):
+#         - `name`: name of bundle store
+#         - `storage_type`: type of storage being used for bundle store (GCP, AWS, etc)
+#         - `storage_format`: the format in which storage is being stored (UNCOMPRESSED, COMPRESSED_V1, etc)
+#         - `url`: a self-referential URL that points to the bundle store.
+#         - `authentication`: key for authentication that the bundle store uses.
+#     """
+#     update = BundleStoreSchema(strict=True, many=True).load(request.json).data[0]
+#     updated_bundle_store = local.model.update_bundle_store(request.user.user_id, uuid, update)
+#     return BundleStoreSchema(many=True).dump(updated_bundle_store).data
+
+
+@get('/bundle_stores/<uuid:re:%s>' % spec_util.UUID_STR, apply=AuthenticatedProtectedPlugin())
+def _fetch_bundle_store(uuid):
+    """
+    Fetch the bundle store corresponding to the specified uuid.
+
+    Returns a single bundle store, with the following parameters:
+    - `uuid`: bundle store UUID
+    - `owner_id`: owner of bundle store
+    - `name`: name of bundle store
+    - `storage_type`: type of storage being used for bundle store (GCP, AWS, etc)
+    - `storage_format`: the format in which storage is being stored (UNCOMPRESSED, COMPRESSED_V1, etc)
+    - `url`: a self-referential URL that points to the bundle store.
+    """
+    bundle_store = local.model.get_bundle_store(request.user.user_id, uuid)
+    data = BundleStoreSchema().dump(bundle_store).data
+
+    # Fetch username instead of user_id for display on the front end.
+    data['data']['attributes']['owner'] = local.model.get_user(
+        user_id=data['data']['attributes']['owner']
+    ).user_name
+    return data
+
+
+@delete('/bundle_stores', apply=AuthenticatedProtectedPlugin())
+def _delete_bundle_stores():
+    """
+    Delete the specified bundle stores.
+    """
+    uuids = get_resource_ids(request.json, 'bundle_stores')
+    for uuid in uuids:
+        local.model.delete_bundle_store(request.user.user_id, uuid)
+
+
 @get('/bundles/<uuid:re:%s>/contents/info/' % spec_util.UUID_STR, name='fetch_bundle_contents_info')
 @get(
     '/bundles/<uuid:re:%s>/contents/info/<path:path>' % spec_util.UUID_STR,
@@ -799,6 +957,8 @@ def _update_bundle_contents_blob(uuid):
     - `use_azure_blob_beta`: (optional) Use Azure Blob Storage to store the bundle.
       Default is False. If CODALAB_ALWAYS_USE_AZURE_BLOB_BETA is set, this parameter
       is disregarded, as Azure Blob Storage will always be used.
+    - `store`: (optional) The name of the bundle store where the bundle should be uploaded to.
+      If unspecified, the CLI will pick the optimal available bundle store.
     """
     check_bundles_have_all_permission(local.model, request.user, [uuid])
     bundle = local.model.get_bundle(uuid)
@@ -808,8 +968,12 @@ def _update_bundle_contents_blob(uuid):
     # Get and validate query parameters
     finalize_on_failure = query_get_bool('finalize_on_failure', default=False)
     finalize_on_success = query_get_bool('finalize_on_success', default=True)
-    use_azure_blob_beta = os.getenv("CODALAB_ALWAYS_USE_AZURE_BLOB_BETA") or query_get_bool(
-        'use_azure_blob_beta', default=False
+    use_azure_blob_beta = query_get_bool('use_azure_blob_beta', default=False)
+    if os.getenv("CODALAB_ALWAYS_USE_AZURE_BLOB_BETA") == "1":
+        use_azure_blob_beta = True
+    store_name = request.query.get('store') or os.getenv('CODALAB_DEFAULT_BUNDLE_STORE_NAME')
+    store = (
+        local.model.get_bundle_store(request.user.user_id, name=store_name) if store_name else None
     )
     final_state = request.query.get('state_on_success', default=State.READY)
     if finalize_on_success and final_state not in State.FINAL_STATES:
@@ -846,6 +1010,7 @@ def _update_bundle_contents_blob(uuid):
                 git=query_get_bool('git', default=False),
                 unpack=query_get_bool('unpack', default=True),
                 use_azure_blob_beta=use_azure_blob_beta,
+                destination_bundle_store=store,
             )
             bundle_link_url = getattr(bundle.metadata, "link_url", None)
             bundle_location = bundle_link_url or local.bundle_store.get_bundle_location(bundle.uuid)
